package p09_klasy;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.*;

class KontoTest {
    private Konto konto;

    @BeforeEach
    void setUp() {
        // Metoda setUp (oznaczona @BeforeEach) wykona się PRZED KAŻDYM TESTEM
        konto = new Konto(1, 1000, new Osoba("Ala", "Kowalska", 30));
    }

    @Test
    void testWplata() {
        // zakładamy, że istnieje konto, na którym jest 1000
        // wykonujemy testowana operację
        konto.wplata(300);
        // za pomocą asercji sprawdzamy stan końcowy
        assertEquals(1300, konto.getSaldo());
    }

    @Test
    void testWyplata() throws BrakSrodkow {
        konto.wyplata(400);
        assertEquals(600, konto.getSaldo());
    }

    // obsługa wyjątków w JUnit

    // wersja JUnit 5:
    @Test
    void testWplataUjemna() {
        // czy na pewno takie wywołanie spowoduje wyjątek?
        assertThrows(IllegalArgumentException.class, () -> konto.wplata(-200));

        // upewniamy się jeszcze, czy saldo nie zostało w tej sytuacji zmienione
        assertEquals(1000, konto.getSaldo());
    }

    // ↓To wklejam gotowe ↓
    // Temat sprawdzania wyjątków w testach.

    // Aby stwierdzić, że w pewnej sytuacji metoda POWINNA skończyć się wyjątkiem,
    // możemy użyć odpowiednio try-catch (testWplataTry)
    // albo użyć dedykowanych rozwiązań JUnit - dalsze przykłady

    @Test
    void testWplataTry() {
        try {
            konto.wplata(-100);
            // Gdyby nie było wyjątku i program doszedłby do tego miejsca,
            // test powinien skończyć się porażką
            fail("Powinien być wyjątek, a nie ma.");
        } catch(IllegalArgumentException e) {
            // Jeśli wyłapiemy wyjątek, możemy sprawdzić w teście jego szczegóły,
            // np. jaki jest message (bardzo rzadko takie rzeczy sprawdza się w testach, ale można):
            assertEquals("Niedodatnia kwota w metodzie wplata", e.getMessage());
        } finally {
            // Możemy też upewnić się, że w przypadku wystąpienia wyjątku stan konta się nie zmienił.
            assertEquals(1000, konto.getSaldo());
        }
    }

    // Analogiczny test można też napisać wykorzystując nową funkcjonalność JUnit 5:
    @Test
    void testWplataUjemna_v1() {
        // assertThrows wykonuje podany fragment kodu (w praktyce podaje się wyrażenie lambda)
        // i sprawdza czy ten kończy się podanym rodzajem wyjątku.
        // Jeśli jest wyjątek - dobrze
        // Jeśli nie ma wyjątku - test failuje

        assertThrows(IllegalArgumentException.class, () -> {
            konto.wplata(-100);
        });

        // Dodatkowo po wykonaniu assertThrows możemy sprawdzić jaki jest stan końcowy,
        // np. czy saldo się nie zmieniło.
        assertEquals(1000, konto.getSaldo());
    }

    // Aby sprawdzić jaki jest message w wyjątku itp, możemy odebrać obiekt wyjątku i sprawdzić bezpośrednio
    @Test
    void testWplataUjemna_v2() {
        IllegalArgumentException exception = assertThrows(IllegalArgumentException.class, () -> {
            konto.wplata(-100);
        });

        assertEquals("Niedodatnia kwota w metodzie wplata", exception.getMessage());
        assertEquals(1000, konto.getSaldo());
    }

    @Test
    void testWyplataUjemna() {
        assertThrows(IllegalArgumentException.class, () -> {
            konto.wyplata(-100);
        });
        assertEquals(1000, konto.getSaldo());
    }

    @Test
    void testBrakSrodkow() {
        assertThrows(BrakSrodkow.class, () -> {
            konto.wyplata(1300);
        });
        assertEquals(1000, konto.getSaldo());
    }

    @Test
    void testPrzelew() throws BrakSrodkow {
        Konto konto2 = new Konto(2, 2000, null);
        konto.przelew(400, konto2);
        assertEquals(600, konto.getSaldo());
        assertEquals(2400, konto2.getSaldo());
    }
}